'''
Created on 23.11.2022

@author: wf
'''
from dataclasses import dataclass
from wikibot.wikiuser import WikiUser
from wikibot.wikiclient import WikiClient
import mwparserfromhell
   
@dataclass
class MediaWikiContext():
    """Class for keeping track of MediaWiki Context"""
    wikiId:str
    wiki_url: str
    context:str 

    def sidif_url(self):
        """
        return the sidif url
        """
        url=f"{self.wiki_url}/index.php/{self.context}Context#sidif"
        return url
    
    def read_sidif(self,withLogin:bool=False)->str:
        """
        
        Read the SiDIF for this Mediawiki context
        
        Args:
            withLogin(bool): if True login
            
        Returns:
            str: the SiDIF
        """
        sidif=None
        wikiusers=WikiUser.getWikiUsers(lenient=True)
        if self.wikiId in wikiusers:
            wikiUser=wikiusers[self.wikiId]
            self.wikiClient=WikiClient.ofWikiId(wikiUser.wikiId)
            if withLogin:
                self.wikiClient.login()
            pageTitle=f"{self.context}"
            page=self.wikiClient.getPage(pageTitle)
            if page.exists:
                markup=page.text()
                mw_code=mwparserfromhell.parse(markup)
                sidif_sections=mw_code.get_sections(matches="sidif")
                if len(sidif_sections)!=1:
                    raise Exception(f"found {len(sidif_sections)} sidif sections but expected exactly 1")
                for node in sidif_sections[0].filter_tags(matches="source"):
                    sidif=str(node.contents)
        return sidif
            
        


'''
Created on 2022-11-23

@author: wf
'''
from tests.basemwtest import BaseMediawikiTest
from meta.mw import MediaWikiContext
from meta.metamodel import Context

class TestMediawiki(BaseMediawikiTest):
    """
    test reading SiDIF from Mediawiki pages
    """
    
    def setUp(self, debug=True, profile=True):
        """
        setUp
        """
        BaseMediawikiTest.setUp(self, debug=debug, profile=profile)
        for wikiId in ["wiki","royals"]:
            self.getWikiUser(wikiId, save=True)

    def test_metamodel_from_wikis(self):
        """
        test reading meta models from wikis
        """
        mw_contexts=[
            MediaWikiContext("wiki",
                "https://wiki.bitplan.com",
                "MetaModel"
            ),  
            MediaWikiContext("royals",
                "http://royal-family.bitplan.com",
                "FamilyContext"
            )
        ]
        
        debug=True
        for mw_context in mw_contexts:
            context,error=Context.fromWikiContext(mw_context,debug=debug)
            self.assertIsNotNone(context)
            if debug:
                print(f"{mw_context.context} has context {context.name}")
            self.assertIsNone(error)
            self.assertEqual(mw_context.context,context.name)


'''
Created on 2022-11-23

@author: wf
'''
from datetime import datetime
from dataclasses import dataclass
from meta.mw import MediaWikiContext
from sidif.sidif import SiDIFParser
from lodstorage.jsonable import JSONAble

class MetaModelElement(JSONAble):
    '''
    a generic MetaModelElement
    
    to handle the technicalities of being a MetaModelElement so that derived
    MetaModelElements can focus on the MetaModel domain specific aspects
    '''
    
class Context(MetaModelElement):
    """
    A Context groups some topics like a Namespace/Package.
    This class provides helper functions and constants to render a Context to corresponding wiki pages
    """
    
    @classmethod
    def getSamples(cls):
        samples = [{
            "name": "MetaModel",
            "since": datetime.strptime("2015-01-23","%Y-%m-%d"),
            "copyright": "2015-2022 BITPlan GmbH",
            "master": "http://master.bitplan.com"
        }]
        return samples
    
    def __init__(self,debug:bool=False):
        """
        constructor
        
        Args:
            debug(bool): if True handle debugging
        """
        self.debug=debug
        
    @classmethod
    def fromDictOfDicts(cls,did:dict)->'Context':
        """
        fill me from the given dict of dicts
        
        Args:
            did(dict): the dict of dicts
            
        Returns:
            Context: the context read
        """
        context=None
        for key,record in did.items():
            isA=record["isA"]
            if isA=="Context":
                context=Context()
                context.fromDict(record)
                pass
        return context
        
    @classmethod
    def fromWikiContext(cls,mw_context:MediaWikiContext,debug:bool=False)->'MetaModel':
        """
        initialize me from the given MediaWiki Context
        
        Args:
            mw_context(MediaWikiContext): the Mediawiki context
            debug(bool): if True handle debugging
            
        Return:
            MetaModel: the metamodel and potential parsing errors
        """
        context=None
        error=None
        if debug:
            print(f"reading sidif for {mw_context.context} from {mw_context.wikiId}")
        sidif=mw_context.read_sidif()
        if sidif is not None:
            sp = SiDIFParser(debug=debug)
            parsed, error = sp.parseText(sidif, mw_context.wikiId)
            if debug:
                sp.printResult(parsed)
            if error is None:
                dif=parsed[0]
                did=dif.toDictOfDicts()
                context=Context.fromDictOfDicts(did)
        return context,error
    
    
@dataclass
class Topic:
    name:str
    pluralName: str
    documentation: str
    context:Context